/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef APE_STREAM_HH
#define APE_STREAM_HH
#include "ap-channels.hh"
#include "ap-triggers.hh"
#include "DatEnv.hh"
#include "OperStateCondList.hh"
#include "TSeries.hh"
#include <vector>
#include <string>
#include <iosfwd>

class ape_stream : public DatEnv {
private:

   //-----------------------------------  2d statistics structure
   struct stat_2d {
      stat_2d(size_t i, size_t j);
      bool operator< (const stat_2d& rhs) const;
      size_t channel_a;
      size_t channel_b;
      size_t min_index;
      size_t trigs_found;
      double trigs_expected;
      double correlation;
      double min_likelihood;
   };

   typedef triggers::gps_t gps_t;
public:
   ape_stream(int argc, const char* argv[]);
   ~ape_stream(void);
   void configure(const std::string& conf);
   void calculate(void);
   void calc_matrices(size_t i, size_t j, int* trig, int* counts) const;
   void calc_stats(const int* trig, const int* counts, stat_2d& stat) const;
   void master_reset();
   void print_config(std::ostream& out) const;
   void ProcessData(void);
   void results(std::ostream& out=std::cout) const;

public:
   enum stats_metric {
      kTriggersOverExpected,
      kLikelihood,
      kTriggers
   };
   std::string  metric_name(stats_metric metric) const;
   stats_metric metric_value(const std::string& metname) const;
   
private:
   std::string _config;
   double      _logCL;
   double      _logCL1D;
   double      _corr_limit;
   size_t      _M;
   bool        _reject_outliers;
   Interval    _stride;
   Interval    _overlap;
   std::vector<std::string> _trig_path;
   
   //-------------------------------------  OSC parameters
   std::string _oscfile;
   std::string _osc_cond;

   //-------------------------------------  html parameters
   std::string _html_directory;
   stats_metric _plot_metric;
   
   //------------------------------------  Channels
   typedef std::vector<channels> chan_vect;
   chan_vect   _channel;
   OperStateCondList _osc;

   //------------------------------------  Trigger reader
   triggers _trig;

   //------------------------------------  Statistics vector
   typedef std::vector<channels::chan_stat> stat1dvec_type;
   typedef stat1dvec_type::iterator stat1dvec_iter;
   stat1dvec_type _stat1d_vector;

   //------------------------------------  Statistics vector
   typedef std::vector<stat_2d> statvec_type;
   typedef statvec_type::iterator statvec_iter;
   statvec_type _stat_vector;
};

//======================================  inline methods
ape_stream::stat_2d::stat_2d(size_t i, size_t j)
   : channel_a(i), channel_b(j), min_index(0), correlation(0),
     min_likelihood(0)
{}

bool 
ape_stream::stat_2d::operator< (const stat_2d& rhs) const {
   //   return min_likelihood < rhs.min_likelihood;
   return trigs_found > rhs.trigs_found;
}

channels::chan_stat::chan_stat(size_t i)
   : channel_id(i), min_index(0), min_likelihood(0)
{}

bool 
channels::chan_stat::operator< (const chan_stat& rhs) const {
   //   return min_likelihood < rhs.min_likelihood;
   return trigs_found > rhs.trigs_found;
}

#endif // !defined(APE_STREAM_HH)
